﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Threading.Tasks;
using System;

public enum ClawState
{
    Swinging,
    Power,
    Launch,
    Retract
}

public class ClawLaunch : MonoBehaviour
{
    ClawState state;

    public Rigidbody2D clawRB;
    public Transform clawTrans;
    public Rope rope;
    public PowerBar powerBar;

    public float swingPos;
    public float power;
    public float launchDirection;

    public bool swingLeft = true;
    public bool powerRise;
    public bool clawOut = false;
    public bool ropeFinished = false;

    void Start()
    {
        GameObject PowerBar = GameObject.Find("PowerBar"); ;

        rope = transform.parent.GetComponent<Rope>();
        clawRB = GetComponent<Rigidbody2D>();
        clawTrans = GetComponent<Transform>();
        powerBar = PowerBar.GetComponent<PowerBar>();

        state = ClawState.Swinging;
    }

    void FixedUpdate()
    {
        switch (state)
        {
            case ClawState.Swinging:
                Swinging();
                break;
            case ClawState.Power:
                SetPower();
                break;
            case ClawState.Launch:
                LaunchClaw();
                break;
            case ClawState.Retract:
                RetractClaw();
                break;
        }    
    }

    public void SwitchState()
    {
        switch (state)
        {
            case ClawState.Swinging:
                state = ClawState.Power;
                break;
            case ClawState.Power:
                state = ClawState.Launch;
                break;
            case ClawState.Launch:
                if (ropeFinished)
                {
                    ropeFinished = false;
                    state = ClawState.Retract;
                }                
                break;
            case ClawState.Retract:
                if (ropeFinished)
                {
                    ropeFinished = false;
                    swingPos = 0;
                    power = 0;
                    state = ClawState.Swinging;
                }   
                break;
        }
    }

    private async Task WaitRope()
    {
        await Task.Delay(TimeSpan.FromSeconds(0.1f));
    }

    void Swinging()
    {
        clawRB.bodyType = RigidbodyType2D.Dynamic;
        power = 0;

        if (swingPos >= 1)
        {
            swingLeft = true;
        }
        else if (swingPos <= -1)
        {
            swingLeft = false;
        }

        if (swingLeft == true)
        {
            swingPos -= Time.deltaTime;
        }
        else
        {
            swingPos += Time.deltaTime;
        }
        clawTrans.position = new Vector3(swingPos, clawTrans.position.y, clawTrans.position.z);

        if (Input.GetKeyDown(KeyCode.Space))
        {
            SwitchState();
        }
    }

    void SetPower()
    {
        if (power >= 21)
        {
            powerRise = false;
        }
        else if (power <= 0)
        {
            powerRise = true;
        }

        if (powerRise)
        {
            power += Time.deltaTime * 10;
            powerBar.SetPower(power);
        }
        else if (!powerRise)
        {
            power -= Time.deltaTime * 10;
            powerBar.SetPower(power);
        }

        clawRB.bodyType = RigidbodyType2D.Static;

        if (Input.GetKeyDown(KeyCode.Space))
        {
            SwitchState();
        }
    }

    async void LaunchClaw()
    {
        if (!clawOut)
        {
            clawOut = true;
            clawRB.bodyType = RigidbodyType2D.Dynamic;
            clawRB.gravityScale = 25;

            for (int i = 0; i < Mathf.Round(power); i++)
            {
                rope.addLink();
                await WaitRope();

                if (i == (Mathf.Round(power) - 1))
                {
                    ropeFinished = true;
                }
            }
        }
        if (Input.GetKeyDown(KeyCode.Space))
        {
            SwitchState();
        }

    }

    async void RetractClaw()
    {
        if (clawOut)
        {
            clawOut = false;
            clawRB.bodyType = RigidbodyType2D.Dynamic;
            clawRB.gravityScale = 1;

            for (int i = 0; i < Mathf.Round(power); i++)
            {
                rope.removeLink();
                await WaitRope();

                if (i == (Mathf.Round(power)-1))
                {
                    ropeFinished = true;
                }
            }
        }
            SwitchState();
    } 
    
}
