﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


public enum EnemyLevel
{
    ONE,
    TWO,
    THREE
}

enum EnemyState
{
    IDLE,
    ATTACKING,
    MOVING
}

public class EnemyController : MonoBehaviour
{
    Animator m_animator;

    public EnemyLevel m_level;
    EnemyState m_state;
    Vector3 m_randomMovementVector;
    static float M_DEFAULT_MOVE_DISTANCE = 5;

    float timer = 0;
    float timerLength;
    bool timerReached = false;

    bool destinationReached = false;

    // Start is called before the first frame update
    void Start()
    {
        m_animator = GetComponent<Animator>();
        if (this.tag == "EnemyLV1")
        {
            SetLevel(EnemyLevel.ONE);
            m_animator.SetBool("level1", true);
        }
        else if (this.tag == "EnemyLV2")
        {
            SetLevel(EnemyLevel.TWO);
            m_animator.SetBool("level2", true);
        }

        m_state = EnemyState.IDLE;
    }

    // Update is called once per frame
    void Update()
    {
        if (!timerReached)
            timer += Time.deltaTime;
        else if (timerReached)
            timerReached = false;

        if (!timerReached && timer > timerLength)
        {
            //Set to false so that We don't run this again
            timerReached = true;
            timer = 0;
        }

        if (this.transform.position == m_randomMovementVector)
        {
            destinationReached = true;
            ChangeEnemyState(EnemyState.IDLE);
        }
    }

    private void FixedUpdate()
    {
        switch (m_state)
        {
            case EnemyState.IDLE:
                timerLength = 0;
                //set  animaion to idle
                destinationReached = false;
                SetRandomMovementVector();
                break;

            case EnemyState.ATTACKING:
                timerLength = 1;
                //set animation to attacking

                //set state to moving
                if (timerReached)
                    ChangeEnemyState(EnemyState.MOVING);

                break;

            case EnemyState.MOVING:
                timerLength = 1;
                if (!destinationReached)
                    MoveEnemy();
                //once destination has been reached, switch state to idle
                if (timerReached)
                    ChangeEnemyState(EnemyState.IDLE);

                break;
        }
    }

    void SetLevel(EnemyLevel level)
    {
        m_level = level;
    }

    void SetRandomMovementVector()
    {
        //give random movement vector a new vector within the travel distance
        m_randomMovementVector = new Vector3(Random.Range(-M_DEFAULT_MOVE_DISTANCE, M_DEFAULT_MOVE_DISTANCE) * Time.deltaTime, Random.Range(-M_DEFAULT_MOVE_DISTANCE, M_DEFAULT_MOVE_DISTANCE) * Time.deltaTime, 0);
    }

    void MoveEnemy()
    {
        SetRandomMovementVector();
        transform.Translate(m_randomMovementVector);
    }

    void ChangeEnemyState(EnemyState newState)
    {
        m_state = newState;
    }

    void OnTriggerEnter2D(Collider2D other)
    {
        ChangeEnemyState(EnemyState.MOVING);
        Invoke("ChangeEnemyState(EnemyState.IDLE)", 2);
    }

    void OnTriggerExit2D(Collider2D other)
    {

    }
}
